@@ -1,23 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -29,31 +27,21 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "gmime-common.h"
 #include "gmime-content-type.h"
 #include "gmime-table-private.h"
+#include "gmime-parse-utils.h"
+
+
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
 
 #define d(x)
 
 
-static int
-param_equal (gconstpointer v, gconstpointer v2)
-{
-	return strcasecmp ((const char *) v, (const char *) v2) == 0;
-}
-
-static guint
-param_hash (gconstpointer key)
-{
-	const char *p = key;
-	guint h = tolower (*p);
-	
-	if (h)
-		for (p += 1; *p != '\0'; p++)
-			h = (h << 5) - h + tolower (*p);
-	
-	return h;
-}
-
 /**
  * g_mime_content_type_new:
  * @type: MIME type (or NULL for "text")
@@ -76,9 +64,9 @@ g_mime_content_type_new (const char *type, const char *subtype)
 	} else {
 		if (type && *type) {
 			mime_type->type = g_strdup (type);
-			if (!strcasecmp (type, "text")) {
+			if (!g_ascii_strcasecmp (type, "text")) {
 				mime_type->subtype = g_strdup ("plain");
-			} else if (!strcasecmp (type, "multipart")) {
+			} else if (!g_ascii_strcasecmp (type, "multipart")) {
 				mime_type->subtype = g_strdup ("mixed");
 			} else {
 				g_free (mime_type->type);
@@ -90,9 +78,9 @@ g_mime_content_type_new (const char *type, const char *subtype)
 			mime_type->subtype = g_strdup ("octet-stream");
 		}
 		
-		g_warning ("Invalid or incomplete type: %s%s%s: defaulting to %s/%s",
-			   type ? type : "", subtype ? "/" : "", subtype ? subtype : "",
-			   mime_type->type, mime_type->subtype);
+		w(g_warning ("Invalid or incomplete type: %s%s%s: defaulting to %s/%s",
+			     type ? type : "", subtype ? "/" : "", subtype ? subtype : "",
+			     mime_type->type, mime_type->subtype));
 	}
 	
 	return mime_type;
@@ -112,7 +100,7 @@ g_mime_content_type_new_from_string (const char *string)
 {
 	GMimeContentType *mime_type;
 	char *type = NULL, *subtype;
-	register const char *inptr;
+	const char *inptr;
 	
 	g_return_val_if_fail (string != NULL, NULL);
 	
@@ -124,40 +112,45 @@ g_mime_content_type_new_from_string (const char *string)
 		inptr++;
 	
 	type = g_strndup (type, (unsigned) (inptr - type));
-	g_strstrip (type);
+	
+	decode_lwsp (&inptr);
 	
 	/* get the subtype */
 	if (*inptr == '/') {
 		inptr++;
 		
-		while (is_lwsp (*inptr))
-			inptr++;
+		decode_lwsp (&inptr);
 		
 		subtype = (char *) inptr;
 		while (*inptr && is_ttoken (*inptr))
 			inptr++;
 		
-		subtype = g_strndup (subtype, (unsigned) (inptr - subtype));
+		if (inptr > subtype)
+			subtype = g_strndup (subtype, (size_t) (inptr - subtype));
+		else
+			subtype = NULL;
+		
+		decode_lwsp (&inptr);
 	} else {
 		subtype = NULL;
 	}
 	
 	mime_type = g_mime_content_type_new (type, subtype);
-	g_free (type);
 	g_free (subtype);
+	g_free (type);
 	
-	while (is_lwsp (*inptr))
+	/* skip past any remaining junk that shouldn't be here... */
+	while (*inptr && *inptr != ';')
 		inptr++;
 	
 	if (*inptr++ == ';' && *inptr) {
 		GMimeParam *p;
 		
 		mime_type->params = g_mime_param_new_from_string (inptr);
-		p = mime_type->params;
-		if (p != NULL) {
-			mime_type->param_hash = g_hash_table_new (param_hash, param_equal);
+		if ((p = mime_type->params) != NULL) {
+			mime_type->param_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 			
-			while (p) {
+			while (p != NULL) {
 				g_hash_table_insert (mime_type->param_hash, p->name, p);
 				p = p->next;
 			}
@@ -237,16 +230,14 @@ g_mime_content_type_is_type (const GMimeContentType *mime_type, const char *type
 	g_return_val_if_fail (type != NULL, FALSE);
 	g_return_val_if_fail (subtype != NULL, FALSE);
 	
-	if (!strcmp (type, "*") || !strcasecmp (mime_type->type, type)) {
+	if (!strcmp (type, "*") || !g_ascii_strcasecmp (mime_type->type, type)) {
 		if (!strcmp (subtype, "*")) {
 			/* special case */
 			return TRUE;
-		} else {
-			if (!strcasecmp (mime_type->subtype, subtype))
-				return TRUE;
-			else
-				return FALSE;
 		}
+		
+		if (!g_ascii_strcasecmp (mime_type->subtype, subtype))
+			return TRUE;
 	}
 	
 	return FALSE;
@@ -271,15 +262,14 @@ g_mime_content_type_set_parameter (GMimeContentType *mime_type, const char *attr
 	g_return_if_fail (value != NULL);
 	
 	if (mime_type->params) {
-		param = g_hash_table_lookup (mime_type->param_hash, attribute);
-		if (param) {
+		if ((param = g_hash_table_lookup (mime_type->param_hash, attribute))) {
 			g_free (param->value);
 			param->value = g_strdup (value);
+			return;
 		}
-	} else {
-		/* hash table may not be initialized */
-		if (!mime_type->param_hash)
-			mime_type->param_hash = g_hash_table_new (param_hash, param_equal);
+	} else if (!mime_type->param_hash) {
+		/* hash table not initialized */
+		mime_type->param_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
 	}
 	
 	if (param == NULL) {
@@ -311,10 +301,8 @@ g_mime_content_type_get_parameter (const GMimeContentType *mime_type, const char
 	if (!mime_type->param_hash)
 		return NULL;
 	
-	param = g_hash_table_lookup (mime_type->param_hash, attribute);
-	
-	if (param)
-		return param->value;
-	else
+	if (!(param = g_hash_table_lookup (mime_type->param_hash, attribute)))
 		return NULL;
+	
+	return param->value;
 }
