@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -135,7 +136,7 @@ filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	inleft = len;
 	
 	do {
-		converted = iconv (charset->cd, &inbuf, &inleft, &outbuf, &outleft);
+		converted = iconv (charset->cd, (ICONV_CONST char **) &inbuf, &inleft, (ICONV_CONST char **) &outbuf, &outleft);
 		if (converted == (size_t) -1) {
 			if (errno == E2BIG || errno == EINVAL)
 				break;
@@ -198,7 +199,7 @@ filter_complete (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	
 	if (inleft > 0) {
 		do {
-			converted = iconv (charset->cd, &inbuf, &inleft, &outbuf, &outleft);
+			converted = iconv (charset->cd, (ICONV_CONST char **) &inbuf, &inleft, (ICONV_CONST char **) &outbuf, &outleft);
 			if (converted != (size_t) -1)
 				continue;
 			
@@ -267,10 +268,10 @@ filter_reset (GMimeFilter *filter)
 
 /**
  * g_mime_filter_charset_new:
- * @from_charset:
- * @to_charset:
+ * @from_charset: charset to convert from
+ * @to_charset: charset to convert to
  *
- * Creates a new GMimeFilterCharset filter.
+ * Creates a new #GMimeFilterCharset filter.
  *
  * Returns a new charset filter.
  **/
@@ -284,7 +285,7 @@ g_mime_filter_charset_new (const char *from_charset, const char *to_charset)
 	if (cd == (iconv_t) -1)
 		return NULL;
 	
-	new = g_object_new (GMIME_TYPE_FILTER_CHARSET, NULL, NULL);
+	new = g_object_new (GMIME_TYPE_FILTER_CHARSET, NULL);
 	new->from_charset = g_strdup (from_charset);
 	new->to_charset = g_strdup (to_charset);
 	new->cd = cd;
