@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -31,11 +32,11 @@
 
 /* text/enriched is rfc1896 */
 
-typedef char * (*EnrichedParamParser) (const char *inptr, int inlen);
+typedef char * (*EnrichedParamParser) (const char *inptr, size_t inlen);
 
-static char *param_parse_colour (const char *inptr, int inlen);
-static char *param_parse_font (const char *inptr, int inlen);
-static char *param_parse_lang (const char *inptr, int inlen);
+static char *param_parse_colour (const char *inptr, size_t inlen);
+static char *param_parse_font (const char *inptr, size_t inlen);
+static char *param_parse_lang (const char *inptr, size_t inlen);
 
 static struct {
 	char *enriched;
@@ -136,7 +137,7 @@ g_mime_filter_enriched_class_init (GMimeFilterEnrichedClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GMimeFilterClass *filter_class = GMIME_FILTER_CLASS (klass);
-	int i;
+	guint i;
 	
 	parent_class = g_type_class_ref (GMIME_TYPE_FILTER);
 	
@@ -204,12 +205,11 @@ static const char *valid_colours[] = {
 #define NUM_VALID_COLOURS  (sizeof (valid_colours) / sizeof (valid_colours[0]))
 
 static char *
-param_parse_colour (const char *inptr, int inlen)
+param_parse_colour (const char *inptr, size_t inlen)
 {
 	const char *inend, *end;
 	guint32 rgb = 0;
-	guint v;
-	int i;
+	guint v, i;
 	
 	for (i = 0; i < NUM_VALID_COLOURS; i++) {
 		if (!g_ascii_strncasecmp (inptr, valid_colours[i], inlen))
@@ -249,7 +249,7 @@ param_parse_colour (const char *inptr, int inlen)
 }
 
 static char *
-param_parse_font (const char *fontfamily, int inlen)
+param_parse_font (const char *fontfamily, size_t inlen)
 {
 	register const char *inptr = fontfamily;
 	const char *inend = inptr + inlen;
@@ -262,7 +262,7 @@ param_parse_font (const char *fontfamily, int inlen)
 }
 
 static char *
-param_parse_lang (const char *lang, int inlen)
+param_parse_lang (const char *lang, size_t inlen)
 {
 	register const char *inptr = lang;
 	const char *inend = inptr + inlen;
@@ -275,9 +275,9 @@ param_parse_lang (const char *lang, int inlen)
 }
 
 static char *
-param_parse (const char *enriched, const char *inptr, int inlen)
+param_parse (const char *enriched, const char *inptr, size_t inlen)
 {
-	int i;
+	guint i;
 	
 	for (i = 0; i < NUM_ENRICHED_TAGS; i++) {
 		if (!g_ascii_strcasecmp (enriched, enriched_tags[i].enriched))
@@ -438,7 +438,7 @@ enriched_to_html (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 			} else {
 				const char *html_tag;
 				char *enriched_tag;
-				int len;
+				size_t len;
 				
 				len = inptr - tag;
 				enriched_tag = g_alloca (len + 1);
