@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -202,18 +203,18 @@ gzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 		
 		memcpy (filter->outbuf, priv->hdr.buf, 10);
 		
-		priv->stream->next_out = filter->outbuf + 10;
+		priv->stream->next_out = (unsigned char *) filter->outbuf + 10;
 		priv->stream->avail_out = filter->outsize - 10;
 		
 		priv->state.zip.wrote_hdr = TRUE;
 	} else {
 		g_mime_filter_set_size (filter, (len * 2) + 12, FALSE);
 		
-		priv->stream->next_out = filter->outbuf;
+		priv->stream->next_out = (unsigned char *) filter->outbuf;
 		priv->stream->avail_out = filter->outsize;
 	}
 	
-	priv->stream->next_in = in;
+	priv->stream->next_in = (unsigned char *) in;
 	priv->stream->avail_in = len;
 	
 	do {
@@ -226,8 +227,8 @@ gzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 			
 			outlen = filter->outsize - priv->stream->avail_out;
 			g_mime_filter_set_size (filter, outlen + (priv->stream->avail_in * 2) + 12, TRUE);
+			priv->stream->next_out = (unsigned char *) filter->outbuf + outlen;
 			priv->stream->avail_out = filter->outsize - outlen;
-			priv->stream->next_out = filter->outbuf + outlen;
 			
 			if (priv->stream->avail_in == 0) {
 				guint32 val;
@@ -246,13 +247,14 @@ gzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 			}
 		} else {
 			if (priv->stream->avail_in > 0)
-				g_mime_filter_backup (filter, priv->stream->next_in, priv->stream->avail_in);
+				g_mime_filter_backup (filter, (char *) priv->stream->next_in,
+						      priv->stream->avail_in);
 			
 			break;
 		}
 	} while (1);
 	
-	priv->crc32 = crc32 (priv->crc32, in, len - priv->stream->avail_in);
+	priv->crc32 = crc32 (priv->crc32, (unsigned char *) in, len - priv->stream->avail_in);
 	priv->isize += len - priv->stream->avail_in;
 	
 	*out = filter->outbuf;
@@ -363,10 +365,10 @@ gunzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	
 	g_mime_filter_set_size (filter, (len * 2) + 12, FALSE);
 	
-	priv->stream->next_in = in;
+	priv->stream->next_in = (unsigned char *) in;
 	priv->stream->avail_in = len - 8;
 	
-	priv->stream->next_out = filter->outbuf;
+	priv->stream->next_out = (unsigned char *) filter->outbuf;
 	priv->stream->avail_out = filter->outsize;
 	
 	do {
@@ -384,13 +386,14 @@ gunzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 			
 			outlen = filter->outsize - priv->stream->avail_out;
 			g_mime_filter_set_size (filter, outlen + (priv->stream->avail_in * 2) + 12, TRUE);
+			priv->stream->next_out = (unsigned char *) filter->outbuf + outlen;
 			priv->stream->avail_out = filter->outsize - outlen;
-			priv->stream->next_out = filter->outbuf + outlen;
 		} else {
 			priv->stream->avail_in += 8;
 			
 			if (priv->stream->avail_in > 0)
-				g_mime_filter_backup (filter, priv->stream->next_in, priv->stream->avail_in);
+				g_mime_filter_backup (filter, (char *) priv->stream->next_in,
+						      priv->stream->avail_in);
 			
 			break;
 		}
@@ -465,7 +468,7 @@ g_mime_filter_gzip_new (GMimeFilterGZipMode mode, int level)
 	GMimeFilterGZip *new;
 	int retval;
 	
-	new = g_object_new (GMIME_TYPE_FILTER_GZIP, NULL, NULL);
+	new = g_object_new (GMIME_TYPE_FILTER_GZIP, NULL);
 	new->mode = mode;
 	new->level = level;
 	
