@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -111,13 +112,17 @@ filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	       char **out, size_t *outlen, size_t *outprespace)
 {
 	GMimeFilterYenc *yenc = (GMimeFilterYenc *) filter;
+	const unsigned char *inbuf;
+	unsigned char *outbuf;
 	size_t newlen = 0;
 	
 	switch (yenc->direction) {
 	case GMIME_FILTER_YENC_DIRECTION_ENCODE:
 		/* won't go to more than 2 * (x + 2) + 62 */
 		g_mime_filter_set_size (filter, (len + 2) * 2 + 62, FALSE);
-		newlen = g_mime_yencode_step (in, len, filter->outbuf, &yenc->state,
+		outbuf = (unsigned char *) filter->outbuf;
+		inbuf = (const unsigned char *) in;
+		newlen = g_mime_yencode_step (inbuf, len, outbuf, &yenc->state,
 					      &yenc->pcrc, &yenc->crc);
 		g_assert (newlen <= (len + 2) * 2 + 62);
 		break;
@@ -188,7 +193,10 @@ filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 		if ((yenc->state & GMIME_YDECODE_STATE_DECODE) && !(yenc->state & GMIME_YDECODE_STATE_END)) {
 			/* all yEnc headers have been found so we can now start decoding */
 			g_mime_filter_set_size (filter, len + 3, FALSE);
-			newlen = g_mime_ydecode_step (in, len, filter->outbuf, &yenc->state, &yenc->pcrc, &yenc->crc);
+			outbuf = (unsigned char *) filter->outbuf;
+			inbuf = (const unsigned char *) in;
+			newlen = g_mime_ydecode_step (inbuf, len, outbuf, &yenc->state,
+						      &yenc->pcrc, &yenc->crc);
 			g_assert (newlen <= len + 3);
 		} else {
 			newlen = 0;
@@ -206,13 +214,17 @@ filter_complete (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 		 char **out, size_t *outlen, size_t *outprespace)
 {
 	GMimeFilterYenc *yenc = (GMimeFilterYenc *) filter;
+	const unsigned char *inbuf;
+	unsigned char *outbuf;
 	size_t newlen = 0;
 	
 	switch (yenc->direction) {
 	case GMIME_FILTER_YENC_DIRECTION_ENCODE:
 		/* won't go to more than 2 * (x + 2) + 62 */
 		g_mime_filter_set_size (filter, (len + 2) * 2 + 62, FALSE);
-		newlen = g_mime_yencode_close (in, len, filter->outbuf, &yenc->state,
+		outbuf = (unsigned char *) filter->outbuf;
+		inbuf = (const unsigned char *) in;
+		newlen = g_mime_yencode_close (inbuf, len, outbuf, &yenc->state,
 					       &yenc->pcrc, &yenc->crc);
 		g_assert (newlen <= (len + 2) * 2 + 62);
 		break;
@@ -220,7 +232,9 @@ filter_complete (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 		if ((yenc->state & GMIME_YDECODE_STATE_DECODE) && !(yenc->state & GMIME_YDECODE_STATE_END)) {
 			/* all yEnc headers have been found so we can now start decoding */
 			g_mime_filter_set_size (filter, len + 3, FALSE);
-			newlen = g_mime_ydecode_step (in, len, filter->outbuf, &yenc->state,
+			outbuf = (unsigned char *) filter->outbuf;
+			inbuf = (const unsigned char *) in;
+			newlen = g_mime_ydecode_step (inbuf, len, outbuf, &yenc->state,
 						      &yenc->pcrc, &yenc->crc);
 			g_assert (newlen <= len + 3);
 		} else {
@@ -266,7 +280,7 @@ g_mime_filter_yenc_new (GMimeFilterYencDirection direction)
 {
 	GMimeFilterYenc *new;
 	
-	new = g_object_new (GMIME_TYPE_FILTER_YENC, NULL, NULL);
+	new = g_object_new (GMIME_TYPE_FILTER_YENC, NULL);
 	new->direction = direction;
 	
 	switch (direction) {
@@ -322,9 +336,9 @@ g_mime_filter_yenc_set_crc (GMimeFilterYenc *yenc, guint32 crc)
  * g_mime_filter_yenc_get_part:
  * @yenc: yEnc filter
  *
- * Gets the part id of the current decoded yEnc stream or -1 on fail.
+ * Gets the part id of the current decoded yEnc stream or %-1 on fail.
  *
- * Returns the part id of the current decoded yEnc stream or -1 on
+ * Returns the part id of the current decoded yEnc stream or %-1 on
  * fail.
  **/
 int
@@ -426,10 +440,10 @@ static const int yenc_crc_table[256] = {
  * @inlen pointed to by @in and writes to @out. Assumes the =ybegin
  * and =ypart lines have already been stripped off.
  *
- * To get the crc32 value of the part, use GMIME_YENCODE_CRC_FINAL
+ * To get the crc32 value of the part, use #GMIME_YENCODE_CRC_FINAL
  * (@pcrc). If there are more parts, you should reuse @crc without
  * re-initializing. Once all parts have been decoded, you may get the
- * combined crc32 value of all the parts using GMIME_YENCODE_CRC_FINAL
+ * combined crc32 value of all the parts using #GMIME_YENCODE_CRC_FINAL
  * (@crc).
  *
  * Returns the number of bytes decoded.
@@ -441,7 +455,7 @@ g_mime_ydecode_step (const unsigned char *in, size_t inlen, unsigned char *out,
 	const register unsigned char *inptr;
 	register unsigned char *outptr;
 	const unsigned char *inend;
-	unsigned char ch;
+	unsigned char c;
 	int ystate;
 	
 	if (*state & GMIME_YDECODE_STATE_END)
@@ -454,37 +468,37 @@ g_mime_ydecode_step (const unsigned char *in, size_t inlen, unsigned char *out,
 	
 	inptr = in;
 	while (inptr < inend) {
-		ch = *inptr++;
+		c = *inptr++;
 		
 		if ((ystate & YENC_NEWLINE_ESCAPE) == YENC_NEWLINE_ESCAPE) {
 			ystate &= ~GMIME_YDECODE_STATE_EOLN;
 			
-			if (ch == 'y') {
+			if (c == 'y') {
 				/* we probably have a =yend here */
 				ystate |= GMIME_YDECODE_STATE_END;
 				break;
 			}
 		}
 		
-		if (ch == '\n') {
+		if (c == '\n') {
 			ystate |= GMIME_YDECODE_STATE_EOLN;
 			continue;
 		}
 		
 		if (ystate & GMIME_YDECODE_STATE_ESCAPE) {
 			ystate &= ~GMIME_YDECODE_STATE_ESCAPE;
-			ch -= 64;
-		} else if (ch == '=') {
+			c -= 64;
+		} else if (c == '=') {
 			ystate |= GMIME_YDECODE_STATE_ESCAPE;
 			continue;
 		}
 		
 		ystate &= ~GMIME_YDECODE_STATE_EOLN;
 		
-		*outptr++ = ch -= 42;
+		*outptr++ = c -= 42;
 		
-		*pcrc = yenc_crc_add (*pcrc, ch);
-		*crc = yenc_crc_add (*crc, ch);
+		*pcrc = yenc_crc_add (*pcrc, c);
+		*crc = yenc_crc_add (*crc, c);
 	}
 	
 	*state = ystate;
@@ -510,7 +524,7 @@ g_mime_ydecode_step (const unsigned char *in, size_t inlen, unsigned char *out,
  * should reuse @state.
  *
  * Along the same lines, @pcrc and @crc should be initialized to
- * GMIME_YENCODE_CRC_INIT before using.
+ * #GMIME_YENCODE_CRC_INIT before using.
  *
  * Returns the number of bytes encoded.
  **/
@@ -522,7 +536,7 @@ g_mime_yencode_step (const unsigned char *in, size_t inlen, unsigned char *out,
 	register unsigned char *outptr;
 	const unsigned char *inend;
 	register int already;
-	unsigned char ch;
+	unsigned char c;
 	
 	inend = in + inlen;
 	outptr = out;
@@ -531,19 +545,19 @@ g_mime_yencode_step (const unsigned char *in, size_t inlen, unsigned char *out,
 	
 	inptr = in;
 	while (inptr < inend) {
-		ch = (*inptr++);
+		c = *inptr++;
 		
-		*pcrc = yenc_crc_add (*pcrc, ch);
-		*crc = yenc_crc_add (*crc, ch);
+		*pcrc = yenc_crc_add (*pcrc, c);
+		*crc = yenc_crc_add (*crc, c);
 		
-		ch += 42;
+		c += 42;
 		
-		if (ch == '\0' || ch == '\t' || ch == '\r' || ch == '\n' || ch == '=') {
+		if (c == '\0' || c == '\t' || c == '\r' || c == '\n' || c == '=') {
 			*outptr++ = '=';
-			*outptr++ = ch + 64;
+			*outptr++ = c + 64;
 			already += 2;
 		} else {
-			*outptr++ = ch;
+			*outptr++ = c;
 			already++;
 		}
 		
