@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -32,6 +33,7 @@
 #include "gmime-utils.h"
 #include "gmime-stream-mem.h"
 #include "gmime-table-private.h"
+#include "gmime-parse-utils.h"
 
 
 static void g_mime_message_class_init (GMimeMessageClass *klass);
@@ -47,10 +49,7 @@ static void message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
-static void message_set_sender (GMimeMessage *message, const char *sender);
-static void message_set_reply_to (GMimeMessage *message, const char *reply_to);
 static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
-static void message_set_subject (GMimeMessage *message, const char *subject);
 
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
@@ -199,8 +198,6 @@ struct _received_token {
 	token_skip_t skip;
 };
 
-extern void decode_lwsp   (const char **in);
-
 static void skip_atom     (const char **in);
 static void skip_domain   (const char **in);
 static void skip_addr     (const char **in);
@@ -417,10 +414,11 @@ static ssize_t
 write_received (GMimeStream *stream, const char *name, const char *value)
 {
 	struct _received_part *parts, *part, *tail;
-	const char *lwsp, *inptr;
+	const char *inptr, *lwsp = NULL;
 	ssize_t nwritten;
 	GString *str;
-	int len, i;
+	size_t len;
+	guint i;
 	
 	while (is_lwsp (*value))
 		value++;
@@ -440,6 +438,7 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 		for (i = 0; i < G_N_ELEMENTS (received_tokens); i++) {
 			if (!strncmp (inptr, received_tokens[i].token, received_tokens[i].len)) {
 				if (inptr > part->start) {
+					g_assert (lwsp != NULL);
 					part->len = lwsp - part->start;
 					
 					part = g_alloca (sizeof (struct _received_part));
@@ -796,8 +795,8 @@ message_get_headers (GMimeObject *object)
 	}
 	
 	g_object_unref (stream);
-	g_byte_array_append (ba, "", 1);
-	str = ba->data;
+	g_byte_array_append (ba, (unsigned char *) "", 1);
+	str = (char *) ba->data;
 	g_byte_array_free (ba, FALSE);
 	
 	return str;
@@ -854,7 +853,7 @@ g_mime_message_new (gboolean pretty_headers)
 	GMimeMessage *message;
 	int i;
 	
-	message = g_object_new (GMIME_TYPE_MESSAGE, NULL, NULL);
+	message = g_object_new (GMIME_TYPE_MESSAGE, NULL);
 	
 	if (pretty_headers) {
 		/* Populate with the "standard" rfc822 headers so we can have a standard order */
@@ -953,12 +952,10 @@ static void
 sync_recipient_header (GMimeMessage *message, const char *type)
 {
 	const InternetAddressList *recipients;
+	char *string;
 	
 	/* sync the specified recipient header */
-	recipients = g_mime_message_get_recipients (message, type);
-	if (recipients) {
-		char *string;
-		
+	if ((recipients = g_mime_message_get_recipients (message, type))) {
 		string = internet_address_list_to_string (recipients, TRUE);
 		g_mime_header_set (GMIME_OBJECT (message)->headers, type, string);
 		g_free (string);
@@ -1010,8 +1007,7 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
 	recipients = g_hash_table_lookup (message->recipients, type);
 	g_hash_table_remove (message->recipients, type);
 	
-	addrlist = internet_address_parse_string (string);
-	if (addrlist) {
+	if ((addrlist = internet_address_parse_string (string))) {
 		recipients = internet_address_list_concat (recipients, addrlist);
 		internet_address_list_destroy (addrlist);
 	}
@@ -1067,12 +1063,56 @@ g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 }
 
 
+static const char *recipient_types[] = {
+	GMIME_RECIPIENT_TYPE_TO,
+	GMIME_RECIPIENT_TYPE_CC,
+	GMIME_RECIPIENT_TYPE_BCC
+};
+
+/**
+ * g_mime_message_get_all_recipients:
+ * @message: MIME Message
+ *
+ * Gets the complete list of recipients for @message.
+ *
+ * Returns a newly allocated #InternetAddressList containing all
+ * recipients of the message.
+ **/
+InternetAddressList *
+g_mime_message_get_all_recipients (GMimeMessage *message)
+{
+	InternetAddressList *list, *tail, *node, *recipients;
+	guint i;
+	
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	
+	list = NULL;
+	tail = (InternetAddressList *) &list;
+	
+	for (i = 0; i < G_N_ELEMENTS (recipient_types); i++) {
+		recipients = g_hash_table_lookup (message->recipients, recipient_types[i]);
+		while (recipients != NULL) {
+			internet_address_ref (recipients->address);
+			node = g_new (InternetAddressList, 1);
+			node->address = recipients->address;
+			node->next = NULL;
+			tail->next = node;
+			tail = node;
+			
+			recipients = recipients->next;
+		}
+	}
+	
+	return list;
+}
+
+
 /**
  * g_mime_message_set_subject:
  * @message: MIME Message
  * @subject: Subject string
  *
- * Set the Subject field on a MIME Message.
+ * Set the unencoded UTF-8 Subject field on a MIME Message.
  **/
 void
 g_mime_message_set_subject (GMimeMessage *message, const char *subject)
@@ -1097,7 +1137,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
  *
  * Gets the message's subject.
  *
- * Returns the Subject field on a MIME Message.
+ * Returns the unencoded UTF-8 Subject field on a MIME Message.
  **/
 const char *
 g_mime_message_get_subject (GMimeMessage *message)
@@ -1314,8 +1354,10 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 	g_object_ref (mime_part);
 	g_mime_header_set_raw (mime_part->headers, NULL);
 	
-	if (message->mime_part)
+	if (message->mime_part) {
+		g_mime_header_set_raw (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
+	}
 	
 	message->mime_part = mime_part;
 }
@@ -1331,7 +1373,7 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
  * WARNING: This interface is deprecated. Use
  * g_mime_object_write_to_stream() instead.
  *
- * Returns -1 on fail.
+ * Returns %-1 on fail.
  **/
 ssize_t
 g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
@@ -1456,6 +1498,8 @@ handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean
  * Attempts to get the body of the message in the preferred format
  * specified by @want_plain.
  *
+ * WARNING: This interface is deprecated.
+ *
  * Returns the prefered form of the message body. Sets the value of
  * @is_html to %TRUE if the part returned is in HTML format, otherwise
  * %FALSE.
