@@ -1,23 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -25,10 +23,10 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
-#include <ctype.h>
 
 #include "gmime-parser.h"
 
@@ -37,10 +35,6 @@
 #include "gmime-multipart.h"
 #include "gmime-part.h"
 
-#ifndef HAVE_ISBLANK
-#define isblank(c) ((c) == ' ' || (c) == '\t')
-#endif /* HAVE_ISBLANK */
-
 #ifdef ENABLE_WARNINGS
 #define w(x) x
 #else
@@ -67,6 +61,7 @@ static GObjectClass *parent_class = NULL;
 #define SCAN_HEAD 128		/* headroom guaranteed to be before each read buffer */
 
 enum {
+	GMIME_PARSER_STATE_ERROR = -1,
 	GMIME_PARSER_STATE_INIT,
 	GMIME_PARSER_STATE_FROM,
 	GMIME_PARSER_STATE_HEADERS,
@@ -82,10 +77,10 @@ struct _GMimeParserPrivate {
 	off_t offset;
 	
 	/* i/o buffers */
-	unsigned char realbuf[SCAN_HEAD + SCAN_BUF + 1];
-	unsigned char *inbuf;
-	unsigned char *inptr;
-	unsigned char *inend;
+	char realbuf[SCAN_HEAD + SCAN_BUF + 1];
+	char *inbuf;
+	char *inptr;
+	char *inend;
 	
 	off_t from_offset;
 	GByteArray *from_line;
@@ -95,9 +90,14 @@ struct _GMimeParserPrivate {
 	gpointer user_data;
 	
 	/* header buffer */
-	unsigned char *headerbuf;
-	unsigned char *headerptr;
-	unsigned int headerleft;
+	char *headerbuf;
+	char *headerptr;
+	guint headerleft;
+	
+	/* raw header buffer */
+	char *rawbuf;
+	char *rawptr;
+	guint rawleft;
 	
 	off_t headers_start;
 	off_t header_start;
@@ -118,13 +118,14 @@ struct _GMimeParserPrivate {
 
 struct _boundary_stack {
 	struct _boundary_stack *parent;
-	unsigned char *boundary;
+	char *boundary;
 	size_t boundarylen;
 	size_t boundarylenfinal;
 	size_t boundarylenmax;
 	off_t content_end;
 };
 
+
 static void
 parser_push_boundary (GMimeParser *parser, const char *boundary)
 {
@@ -185,7 +186,7 @@ header_raw_find (struct _header_raw *headers, const char *name, off_t *offset)
 	
 	h = headers;
 	while (h) {
-		if (!strcasecmp (h->name, name)) {
+		if (!g_ascii_strcasecmp (h->name, name)) {
 			if (offset)
 				*offset = h->offset;
 			return h->value;
@@ -228,7 +229,7 @@ g_mime_parser_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeParser),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_parser_init,
 		};
 		
@@ -305,6 +306,10 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->headerptr = priv->headerbuf;
 	priv->headerleft = SCAN_HEAD;
 	
+	priv->rawbuf = g_malloc (SCAN_HEAD + 1);
+	priv->rawptr = priv->rawbuf;
+	priv->rawleft = SCAN_HEAD;
+	
 	priv->headers_start = -1;
 	priv->header_start = -1;
 	
@@ -327,8 +332,8 @@ parser_close (GMimeParser *parser)
 	
 	g_byte_array_free (priv->from_line, TRUE);
 	
-	if (priv->headerbuf)
-		g_free (priv->headerbuf);
+	g_free (priv->headerbuf);
+	g_free (priv->rawbuf);
 	
 	header_raw_clear (&priv->headers);
 	
@@ -357,7 +362,7 @@ g_mime_parser_new (void)
 {
 	GMimeParser *parser;
 	
-	parser = g_object_new (GMIME_TYPE_PARSER, NULL, NULL);
+	parser = g_object_new (GMIME_TYPE_PARSER, NULL);
 	
 	return parser;
 }
@@ -419,7 +424,7 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
  * g_mime_parser_get_persist_stream:
  * @parser: MIME parser object
  *
- * Gets whether or not the underlying stream is persistant.
+ * Gets whether or not the underlying stream is persistent.
  *
  * Returns %TRUE if the @parser will leave the content on disk or
  * %FALSE if it will load the content into memory.
@@ -438,10 +443,10 @@ g_mime_parser_get_persist_stream (GMimeParser *parser)
  * @parser: MIME parser object
  * @persist: persist attribute
  *
- * Sets whether or not the @parser's underlying stream is persistant.
+ * Sets whether or not the @parser's underlying stream is persistent.
  *
  * If @persist is %TRUE, the @parser will attempt to construct
- * messages/parts whos content will remain on disk rather than being
+ * messages/parts whose content will remain on disk rather than being
  * loaded into memory so as to reduce memory usage. This is the default.
  *
  * If @persist is %FALSE, the @parser will always load message content
@@ -574,9 +579,9 @@ static ssize_t
 parser_fill (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	unsigned char *inbuf, *inptr, *inend;
-	size_t inlen, atleast = SCAN_HEAD;
+	char *inbuf, *inptr, *inend;
 	ssize_t nread;
+	size_t inlen;
 	
 	inbuf = priv->inbuf;
 	inptr = priv->inptr;
@@ -585,11 +590,6 @@ parser_fill (GMimeParser *parser)
 	
 	g_assert (inptr <= inend);
 	
-	atleast = MAX (atleast, priv->bounds ? priv->bounds->boundarylenmax : 0);
-	
-	if (inlen > atleast)
-		return inlen;
-	
 	/* attempt to align 'inend' with realbuf + SCAN_HEAD */
 	if (inptr >= inbuf) {
 		inbuf -= inlen < SCAN_HEAD ? inlen : SCAN_HEAD;
@@ -622,7 +622,7 @@ parser_fill (GMimeParser *parser)
 
 
 static off_t
-parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
+parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
 {
 	if (priv->offset == -1)
 		return -1;
@@ -676,8 +676,9 @@ static int
 parser_step_from (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register unsigned char *inptr;
-	unsigned char *start, *inend;
+	register char *inptr;
+	char *start, *inend;
+	ssize_t left = 0;
 	size_t len;
 	
 	g_byte_array_set_size (priv->from_line, 0);
@@ -688,8 +689,12 @@ parser_step_from (GMimeParser *parser)
 	
 	do {
 	refill:
-		if (parser_fill (parser) <= 0)
-			break;
+		if (parser_fill (parser) <= left) {
+			/* failed to find a From line; EOF reached */
+			priv->state = GMIME_PARSER_STATE_ERROR;
+			priv->inptr = priv->inend;
+			return -1;
+		}
 		
 		inptr = priv->inptr;
 		inend = priv->inend;
@@ -701,7 +706,8 @@ parser_step_from (GMimeParser *parser)
 				inptr++;
 			
 			if (inptr + 1 >= inend) {
-				/* we don't have enough data */
+				/* we don't have enough data; if we can't get more we have to bail */
+				left = inend - start;
 				priv->inptr = start;
 				goto refill;
 			}
@@ -711,12 +717,13 @@ parser_step_from (GMimeParser *parser)
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
 				priv->from_offset = parser_offset (priv, start);
-				g_byte_array_append (priv->from_line, start, len);
+				g_byte_array_append (priv->from_line, (unsigned char *) start, len);
 				goto got_from;
 			}
 		}
 		
 		priv->inptr = inptr;
+		left = 0;
 	} while (1);
 	
  got_from:
@@ -728,7 +735,7 @@ parser_step_from (GMimeParser *parser)
 	return 0;
 }
 
-#define header_backup(priv, start, len) G_STMT_START {                    \
+#define header_append(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
 		size_t hlen, hoff;                                        \
 		                                                          \
@@ -748,9 +755,34 @@ parser_step_from (GMimeParser *parser)
 	priv->headerleft -= len;                                          \
 } G_STMT_END
 
+#define raw_header_append(priv, start, len) G_STMT_START {                \
+	if (priv->rawleft <= len) {                                       \
+		size_t hlen, hoff;                                        \
+		                                                          \
+		hlen = hoff = priv->rawptr - priv->rawbuf;                \
+		hlen = hlen ? hlen : 1;                                   \
+		                                                          \
+		while (hlen < hoff + len)                                 \
+			hlen <<= 1;                                       \
+		                                                          \
+		priv->rawbuf = g_realloc (priv->rawbuf, hlen + 1);        \
+		priv->rawptr = priv->rawbuf + hoff;                       \
+		priv->rawleft = hlen - hoff;                              \
+	}                                                                 \
+	                                                                  \
+	memcpy (priv->rawptr, start, len);                                \
+	priv->rawptr += len;                                              \
+	priv->rawleft -= len;                                             \
+} G_STMT_END
+
+#define raw_header_reset(priv) G_STMT_START {                             \
+	priv->rawleft += priv->rawptr - priv->rawbuf;                     \
+	priv->rawptr = priv->rawbuf;                                      \
+} G_STMT_END
+
 #define header_parse(parser, priv, hend) G_STMT_START {                   \
-	register unsigned char *colon;                                    \
 	struct _header_raw *header;                                       \
+	register char *colon;                                             \
 	size_t hlen;                                                      \
 	                                                                  \
 	header = g_new (struct _header_raw, 1);                           \
@@ -791,10 +823,11 @@ static int
 parser_step_headers (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register unsigned char *inptr;
-	unsigned char *start, *inend;
 	struct _header_raw *hend;
-	size_t len = 0;
+	register char *inptr;
+	char *start, *inend;
+	ssize_t left = 0;
+	size_t len;
 	
 	priv->midline = FALSE;
 	hend = (struct _header_raw *) &priv->headers;
@@ -806,7 +839,7 @@ parser_step_headers (GMimeParser *parser)
 	
 	do {
 	refill:
-		if (parser_fill (parser) <= len)
+		if (parser_fill (parser) <= left)
 			break;
 		
 		inptr = priv->inptr;
@@ -822,50 +855,59 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
+			len = (size_t) (inptr - start);
+			raw_header_append (priv, start, len);
+			
 			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
-				priv->inptr = start;
-				len = inend - start;
+				header_append (priv, start, len);
+				priv->midline = TRUE;
+				left = inend - inptr;
+				priv->inptr = inptr;
 				goto refill;
 			}
 			
 			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && inptr == start)
+			if (!priv->midline && (inptr == start || (len == 1 && *start == '\r')))
 				goto headers_end;
 			
-			len = inptr - start;
-			header_backup (priv, start, len);
+			if (inptr > start && inptr[-1] == '\r')
+				len--;
 			
-			if (inptr < inend) {
-				/* inptr has to be less than inend - 1 */
-				inptr++;
-				if (*inptr == ' ' || *inptr == '\t') {
-					priv->midline = TRUE;
-				} else {
-					priv->midline = FALSE;
-					header_parse (parser, priv, hend);
-					priv->header_start = parser_offset (priv, inptr);
-				}
-			} else {
+			header_append (priv, start, len);
+			
+			/* inptr has to be less than inend - 1 */
+			raw_header_append (priv, inptr, 1);
+			inptr++;
+			
+			if (*inptr == ' ' || *inptr == '\t') {
 				priv->midline = TRUE;
+			} else {
+				priv->midline = FALSE;
+				header_parse (parser, priv, hend);
+				priv->header_start = parser_offset (priv, inptr);
 			}
 		}
 		
+		left = inend - inptr;
 		priv->inptr = inptr;
-		len = inend - inptr;
 	} while (1);
 	
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
-	header_backup (priv, inptr, inend - inptr);
+	len = (size_t) (inend - inptr);
+	header_append (priv, inptr, len);
+	raw_header_append (priv, inptr, len);
 	
  headers_end:
 	
 	if (priv->headerptr > priv->headerbuf)
 		header_parse (parser, priv, hend);
 	
+	*priv->rawptr = '\0';
+	
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
 	
 	g_assert (inptr <= priv->inend);
@@ -908,6 +950,8 @@ parser_step (GMimeParser *parser)
 		case GMIME_PARSER_STATE_HEADERS:
 			parser_step_headers (parser);
 			break;
+		case GMIME_PARSER_STATE_ERROR:
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -923,8 +967,8 @@ static void
 parser_skip_line (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register unsigned char *inptr;
-	unsigned char *inend;
+	register char *inptr;
+	char *inend;
 	
 	inptr = priv->inptr;
 	
@@ -958,9 +1002,9 @@ enum {
 	FOUND_END_BOUNDARY
 };
 
-#define content_save(content, start, len) G_STMT_START {   \
-	if (content)                                       \
-		g_byte_array_append (content, start, len); \
+#define content_save(content, start, len) G_STMT_START {                     \
+	if (content)                                                         \
+		g_byte_array_append (content, (unsigned char *) start, len); \
 } G_STMT_END
 
 #define possible_boundary(scan_from, start, len)                                      \
@@ -968,9 +1012,12 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
+check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
 {
-	off_t offset = parser_offset (priv, (unsigned char *) start);
+	off_t offset = parser_offset (priv, start);
+	
+	if (start[len - 1] == '\r')
+		len--;
 	
 	if (possible_boundary (priv->scan_from, start, len)) {
 		struct _boundary_stack *s;
@@ -1016,11 +1063,11 @@ check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, si
  **/
 
 static int
-parser_scan_content (GMimeParser *parser, GByteArray *content)
+parser_scan_content (GMimeParser *parser, GByteArray *content, guint *crlf)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	register unsigned char *inptr;
-	unsigned char *start, *inend;
+	register char *inptr;
+	char *start, *inend;
 	size_t nleft, len;
 	int found = 0;
 	
@@ -1046,7 +1093,8 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 		/* Note: see optimization comment [1] */
 		*inend = '\n';
 		
-		if (priv->midline && inend - inptr == nleft)
+		len = (size_t) (inend - inptr);
+		if (priv->midline && len == nleft)
 			found = FOUND_EOS;
 		
 		priv->midline = FALSE;
@@ -1057,12 +1105,13 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 			while (*inptr != '\n')
 				inptr++;
 			
-			len = inptr - start;
+			len = (size_t) (inptr - start);
 			
 			if (inptr < inend) {
-				inptr++;
 				if ((found = check_boundary (priv, start, len)))
 					goto boundary;
+				
+				inptr++;
 				len++;
 			} else {
 				/* didn't find an end-of-line */
@@ -1091,6 +1140,15 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 	/* don't chew up the boundary */
 	priv->inptr = start;
 	
+	if (found != FOUND_EOS) {
+		if (inptr[-1] == '\r')
+			*crlf = 2;
+		else
+			*crlf = 1;
+	} else {
+		*crlf = 0;
+	}
+	
 	return found;
 }
 
@@ -1103,19 +1161,20 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
 	off_t start, end;
+	guint crlf;
 	
 	if (priv->persist_stream && priv->seekable)
 		start = parser_offset (priv, NULL);
 	else
 		content = g_byte_array_new ();
 	
-	*found = parser_scan_content (parser, content);
+	*found = parser_scan_content (parser, content, &crlf);
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
-			end = parser_offset (priv, NULL) - 1;
+			end = parser_offset (priv, NULL) - crlf;
 		else
-			g_byte_array_set_size (content, MAX (content->len - 1, 0));
+			g_byte_array_set_size (content, MAX (content->len - crlf, 0));
 	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (priv, NULL);
 	}
@@ -1155,19 +1214,17 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 		header = header->next;
 	}
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
-	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*")) {
+	if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, found);
 	} else {
 		object = parser_construct_leaf_part (parser, content_type, found);
 	}
 	
-	g_mime_message_set_mime_part (message, object);
-	g_object_unref (object);
+	message->mime_part = object;
 	
 	g_mime_message_part_set_message (mpart, message);
 	g_object_unref (message);
@@ -1179,17 +1236,11 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _header_raw *header;
 	GMimeObject *object;
-	const char *ctype;
 	
 	/* get the headers */
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
-	if (!content_type) {
-		if (!(content_type = parser_content_type (parser)))
-			content_type = g_mime_content_type_new ("text", "plain");
-	}
-	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
 	header = priv->headers;
 	while (header) {
@@ -1199,9 +1250,13 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_content_type_destroy (object->content_type);
+	if (object->content_type)
+		g_mime_content_type_destroy (object->content_type);
 	object->content_type = content_type;
 	
+	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	raw_header_reset (priv);
+	
 	/* skip empty line after headers */
 	parser_skip_line (parser);
 	
@@ -1214,27 +1269,54 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	return object;
 }
 
+static void
+crlf2lf (char *in)
+{
+	register char *inptr = in;
+	register char *outptr;
+	
+	while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
+		inptr++;
+	
+	if (*inptr == '\0')
+		return;
+	
+	outptr = inptr++;
+	
+	while (*inptr != '\0') {
+		while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
+			*outptr++ = *inptr++;
+		
+		if (*inptr == '\r')
+			inptr++;
+	}
+	
+	*outptr = '\0';
+}
+
 static int
 parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboolean preface)
 {
 	GByteArray *buffer;
-	const char *face;
+	guint crlf;
+	char *face;
 	int found;
 	
 	buffer = g_byte_array_new ();
-	found = parser_scan_content (parser, buffer);
+	found = parser_scan_content (parser, buffer, &crlf);
 	
-	/* last '\n' belongs to the boundary */
-	if (buffer->len) {
-		buffer->data[buffer->len - 1] = '\0';
-		face = buffer->data;
-	} else
-		face = NULL;
-	
-	if (preface)
-		g_mime_multipart_set_preface (multipart, face);
-	else
-		g_mime_multipart_set_postface (multipart, face);
+	if (buffer->len >= crlf) {
+		/* last '\n' belongs to the boundary */
+		g_byte_array_set_size (buffer, buffer->len + 1);
+		buffer->data[buffer->len - crlf - 1] = '\0';
+		face = (char *) buffer->data;
+		crlf2lf (face);
+		
+		if (preface)
+			g_mime_multipart_set_preface (multipart, face);
+		else
+			g_mime_multipart_set_postface (multipart, face);
+	}
 	
 	g_byte_array_free (buffer, TRUE);
 	
@@ -1258,8 +1340,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		/* get the headers */
 		parser_step_headers (parser);
 		
-		content_type = parser_content_type (parser);
-		if (!content_type)
+		if (!(content_type = parser_content_type (parser)))
 			content_type = g_mime_content_type_new ("text", "plain");
 		
 		parser_unstep (parser);
@@ -1306,9 +1387,13 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_content_type_destroy (object->content_type);
+	if (object->content_type)
+		g_mime_content_type_destroy (object->content_type);
 	object->content_type = content_type;
 	
+	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	raw_header_reset (priv);
+	
 	multipart = (GMimeMultipart *) object;
 	
 	/* skip empty line after headers */
@@ -1351,8 +1436,7 @@ parser_construct_part (GMimeParser *parser)
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1392,16 +1476,19 @@ parser_construct_message (GMimeParser *parser)
 	int content_length = -1;
 	GMimeMessage *message;
 	GMimeObject *object;
-	int found;
+	int state, found;
 	
 	/* get the headers (and, optionally, the from-line) */
-	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
+	while ((state = parser_step (parser)) != GMIME_PARSER_STATE_ERROR && state != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
+	if (state == GMIME_PARSER_STATE_ERROR)
+		return NULL;
+	
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		if (priv->respect_content_length && !strcasecmp (header->name, "Content-Length"))
+		if (priv->respect_content_length && !g_ascii_strcasecmp (header->name, "Content-Length"))
 			content_length = strtoul (header->value, NULL, 10);
 		
 		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
@@ -1414,8 +1501,7 @@ parser_construct_message (GMimeParser *parser)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1425,8 +1511,7 @@ parser_construct_message (GMimeParser *parser)
 		object = parser_construct_leaf_part (parser, content_type, &found);
 	}
 	
-	g_mime_message_set_mime_part (message, object);
-	g_object_unref (object);
+	message->mime_part = object;
 	
 	if (priv->scan_from) {
 		priv->state = GMIME_PARSER_STATE_FROM;
@@ -1476,7 +1561,7 @@ g_mime_parser_get_from (GMimeParser *parser)
 		return NULL;
 	
 	if (priv->from_line->len)
-		return g_strndup (priv->from_line->data, priv->from_line->len);
+		return g_strndup ((char *) priv->from_line->data, priv->from_line->len);
 	
 	return NULL;
 }
