@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -79,7 +80,7 @@ g_mime_part_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimePart),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_part_init,
 		};
 		
@@ -429,12 +430,12 @@ mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * text/plain.
  **/
 GMimePart *
-g_mime_part_new ()
+g_mime_part_new (void)
 {
 	GMimePart *mime_part;
 	GMimeContentType *type;
 	
-	mime_part = g_object_new (GMIME_TYPE_PART, NULL, NULL);
+	mime_part = g_object_new (GMIME_TYPE_PART, NULL);
 	
 	type = g_mime_content_type_new ("text", "plain");
 	g_mime_object_set_content_type (GMIME_OBJECT (mime_part), type);
@@ -458,7 +459,7 @@ g_mime_part_new_with_type (const char *type, const char *subtype)
 	GMimePart *mime_part;
 	GMimeContentType *content_type;
 	
-	mime_part = g_object_new (GMIME_TYPE_PART, NULL, NULL);
+	mime_part = g_object_new (GMIME_TYPE_PART, NULL);
 	
 	content_type = g_mime_content_type_new (type, subtype);
 	g_mime_object_set_content_type (GMIME_OBJECT (mime_part), content_type);
@@ -580,7 +581,7 @@ g_mime_part_get_content_id (GMimePart *mime_part)
 /**
  * g_mime_part_set_content_md5:
  * @mime_part: Mime part
- * @content_md5: content md5 or NULL to generate the md5 digest.
+ * @content_md5: content md5 or %NULL to generate the md5 digest.
  *
  * Set the content md5 for the specified mime part.
  **/
@@ -592,7 +593,8 @@ g_mime_part_set_content_md5 (GMimePart *mime_part, const char *content_md5)
 	GMimeStreamFilter *filtered_stream;
 	GMimeFilter *md5_filter;
 	GMimeStream *stream;
-	int state, save;
+	guint32 save = 0;
+	int state = 0;
 	size_t len;
 	
 	g_return_if_fail (GMIME_IS_PART (mime_part));
@@ -628,10 +630,9 @@ g_mime_part_set_content_md5 (GMimePart *mime_part, const char *content_md5)
 		g_mime_filter_md5_get_digest ((GMimeFilterMd5 *) md5_filter, digest);
 		g_object_unref (md5_filter);
 		
-		state = save = 0;
 		len = g_mime_utils_base64_encode_close (digest, 16, b64digest, &state, &save);
 		b64digest[len] = '\0';
-		g_strstrip (b64digest);
+		g_strstrip ((char *) b64digest);
 		
 		content_md5 = (const char *) b64digest;
 	}
@@ -647,8 +648,8 @@ g_mime_part_set_content_md5 (GMimePart *mime_part, const char *content_md5)
  *
  * Verify the content md5 for the specified mime part.
  *
- * Returns TRUE if the md5 is valid or FALSE otherwise. Note: will
- * return FALSE if the mime part does not contain a Content-MD5.
+ * Returns %TRUE if the md5 is valid or %FALSE otherwise. Note: will
+ * return %FALSE if the mime part does not contain a Content-MD5.
  **/
 gboolean
 g_mime_part_verify_content_md5 (GMimePart *mime_part)
@@ -658,7 +659,8 @@ g_mime_part_verify_content_md5 (GMimePart *mime_part)
 	GMimeStreamFilter *filtered_stream;
 	GMimeFilter *md5_filter;
 	GMimeStream *stream;
-	int state, save;
+	guint32 save = 0;
+	int state = 0;
 	size_t len;
 	
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), FALSE);
@@ -693,12 +695,11 @@ g_mime_part_verify_content_md5 (GMimePart *mime_part)
 	g_mime_filter_md5_get_digest ((GMimeFilterMd5 *) md5_filter, digest);
 	g_object_unref (md5_filter);
 	
-	state = save = 0;
 	len = g_mime_utils_base64_encode_close (digest, 16, b64digest, &state, &save);
 	b64digest[len] = '\0';
-	g_strstrip (b64digest);
+	g_strstrip ((char *) b64digest);
 	
-	return !strcmp (b64digest, mime_part->content_md5);
+	return !strcmp ((char *) b64digest, mime_part->content_md5);
 }
 
 
@@ -823,10 +824,10 @@ g_mime_part_set_encoding (GMimePart *mime_part, GMimePartEncodingType encoding)
  *
  * Returns the content encoding for the specified mime part. The
  * return value will be one of the following:
- * GMIME_PART_ENCODING_DEFAULT, GMIME_PART_ENCODING_7BIT,
- * GMIME_PART_ENCODING_8BIT, GMIME_PART_ENCODING_BINARY,
- * GMIME_PART_ENCODING_BASE64, GMIME_PART_ENCODING_QUOTEDPRINTABLE
- * or GMIME_PART_ENCODING_UUENCODE.
+ * #GMIME_PART_ENCODING_DEFAULT, #GMIME_PART_ENCODING_7BIT,
+ * #GMIME_PART_ENCODING_8BIT, #GMIME_PART_ENCODING_BINARY,
+ * #GMIME_PART_ENCODING_BASE64, #GMIME_PART_ENCODING_QUOTEDPRINTABLE
+ * or #GMIME_PART_ENCODING_UUENCODE.
  **/
 GMimePartEncodingType
 g_mime_part_get_encoding (GMimePart *mime_part)
@@ -844,11 +845,11 @@ g_mime_part_get_encoding (GMimePart *mime_part)
  * Gets the string value of the content encoding.
  *
  * Returns the encoding type as a string. Available
- * values for the encoding are: GMIME_PART_ENCODING_DEFAULT,
- * GMIME_PART_ENCODING_7BIT, GMIME_PART_ENCODING_8BIT,
- * GMIME_PART_ENCODING_BINARY, GMIME_PART_ENCODING_BASE64,
- * GMIME_PART_ENCODING_QUOTEDPRINTABLE and
- * GMIME_PART_ENCODING_UUENCODE.
+ * values for the encoding are: #GMIME_PART_ENCODING_DEFAULT,
+ * #GMIME_PART_ENCODING_7BIT, #GMIME_PART_ENCODING_8BIT,
+ * #GMIME_PART_ENCODING_BINARY, #GMIME_PART_ENCODING_BASE64,
+ * #GMIME_PART_ENCODING_QUOTEDPRINTABLE and
+ * #GMIME_PART_ENCODING_UUENCODE.
  **/
 const char *
 g_mime_part_encoding_to_string (GMimePartEncodingType encoding)
@@ -880,12 +881,12 @@ g_mime_part_encoding_to_string (GMimePartEncodingType encoding)
  * Gets the content encoding enumeration value based on the input
  * string.
  *
- * Returns the encoding string as a GMimePartEncodingType.  Available
- * values for the encoding are: GMIME_PART_ENCODING_DEFAULT,
- * GMIME_PART_ENCODING_7BIT, GMIME_PART_ENCODING_8BIT,
- * GMIME_PART_ENCODING_BINARY, GMIME_PART_ENCODING_BASE64,
- * GMIME_PART_ENCODING_QUOTEDPRINTABLE and
- * GMIME_PART_ENCODING_UUENCODE.
+ * Returns the encoding string as a #GMimePartEncodingType.  Available
+ * values for the encoding are: #GMIME_PART_ENCODING_DEFAULT,
+ * #GMIME_PART_ENCODING_7BIT, #GMIME_PART_ENCODING_8BIT,
+ * #GMIME_PART_ENCODING_BINARY, #GMIME_PART_ENCODING_BASE64,
+ * #GMIME_PART_ENCODING_QUOTEDPRINTABLE and
+ * #GMIME_PART_ENCODING_UUENCODE.
  **/
 GMimePartEncodingType
 g_mime_part_encoding_from_string (const char *encoding)
@@ -900,6 +901,8 @@ g_mime_part_encoding_from_string (const char *encoding)
 		return GMIME_PART_ENCODING_BASE64;
 	else if (!g_ascii_strcasecmp (encoding, "quoted-printable"))
 		return GMIME_PART_ENCODING_QUOTEDPRINTABLE;
+	else if (!g_ascii_strcasecmp (encoding, "uuencode"))
+		return GMIME_PART_ENCODING_UUENCODE;
 	else if (!g_ascii_strcasecmp (encoding, "x-uuencode"))
 		return GMIME_PART_ENCODING_UUENCODE;
 	else
@@ -997,7 +1000,7 @@ g_mime_part_add_content_disposition_parameter (GMimePart *mime_part, const char
  * @attribute, or %NULL if the parameter does not exist.
  *
  * Returns the value of a previously defined content-disposition
- * parameter specified by #name.
+ * parameter specified by @attribute.
  **/
 const char *
 g_mime_part_get_content_disposition_parameter (GMimePart *mime_part, const char *attribute)
@@ -1272,22 +1275,22 @@ g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
 		g_object_unref (cache);
 		
 		*len = buf->len;
-		retval = buf->data;
+		retval = (char *) buf->data;
 	} else {
 		GByteArray *buf = GMIME_STREAM_MEM (stream)->buffer;
-		int start_index = 0;
-		int end_index = buf->len;
+		off_t end_index = (off_t) buf->len;
+		off_t start_index = 0;
 		
 		/* check boundaries */
 		if (stream->bound_start >= 0)
-			start_index = CLAMP (stream->bound_start, 0, buf->len);
+			start_index = CLAMP (stream->bound_start, 0, (off_t) buf->len);
 		if (stream->bound_end >= 0)
-			end_index = CLAMP (stream->bound_end, 0, buf->len);
+			end_index = CLAMP (stream->bound_end, 0, (off_t) buf->len);
 		if (end_index < start_index)
 			end_index = start_index;
 		
 		*len = end_index - start_index;
-		retval = buf->data + start_index;
+		retval = (char *) buf->data + start_index;
 	}
 	
 	return retval;
@@ -1304,7 +1307,7 @@ g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
  * WARNING: This interface is deprecated. Use
  * g_mime_object_write_to_stream() instead.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_part_write_to_stream (GMimePart *mime_part, GMimeStream *stream)
