@@ -1,23 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -34,7 +32,7 @@ static void g_mime_stream_mem_init (GMimeStreamMem *stream, GMimeStreamMemClass
 static void g_mime_stream_mem_finalize (GObject *object);
 
 static ssize_t stream_read (GMimeStream *stream, char *buf, size_t len);
-static ssize_t stream_write (GMimeStream *stream, char *buf, size_t len);
+static ssize_t stream_write (GMimeStream *stream, const char *buf, size_t len);
 static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
@@ -62,7 +60,7 @@ g_mime_stream_mem_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamMem),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_mem_init,
 		};
 		
@@ -123,7 +121,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	n = MIN (bound_end - stream->position, (off_t) len);
 	if (n > 0) {
@@ -138,7 +136,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 }
 
 static ssize_t
-stream_write (GMimeStream *stream, char *buf, size_t len)
+stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	off_t bound_end;
@@ -193,7 +191,7 @@ stream_eos (GMimeStream *stream)
 	
 	g_return_val_if_fail (mem->buffer != NULL, TRUE);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	return stream->position >= bound_end;
 }
@@ -203,11 +201,7 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
-	
-	stream->position = stream->bound_start;
-	
-	return 0;
+	return mem->buffer ? 0 : -1;
 }
 
 static off_t
@@ -218,7 +212,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -260,7 +254,7 @@ stream_length (GMimeStream *stream)
 	
 	g_return_val_if_fail (mem->buffer != NULL, -1);
 	
-	bound_end = stream->bound_end != -1 ? stream->bound_end : mem->buffer->len;
+	bound_end = stream->bound_end != -1 ? stream->bound_end : (off_t) mem->buffer->len;
 	
 	return bound_end - stream->bound_start;
 }
@@ -270,7 +264,7 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 {
 	GMimeStreamMem *mem;
 	
-	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL, NULL);
+	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL);
 	mem->owner = FALSE;
 	mem->buffer = GMIME_STREAM_MEM (stream)->buffer;
 	
@@ -292,7 +286,7 @@ g_mime_stream_mem_new (void)
 {
 	GMimeStreamMem *mem;
 	
-	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL, NULL);
+	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL);
 	mem->owner = TRUE;
 	mem->buffer = g_byte_array_new ();
 	
@@ -315,7 +309,7 @@ g_mime_stream_mem_new_with_byte_array (GByteArray *array)
 {
 	GMimeStreamMem *mem;
 	
-	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL, NULL);
+	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL);
 	mem->owner = TRUE;
 	mem->buffer = array;
 	
@@ -340,11 +334,11 @@ g_mime_stream_mem_new_with_buffer (const char *buffer, size_t len)
 {
 	GMimeStreamMem *mem;
 	
-	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL, NULL);
+	mem = g_object_new (GMIME_TYPE_STREAM_MEM, NULL);
 	mem->owner = TRUE;
 	mem->buffer = g_byte_array_new ();
 	
-	g_byte_array_append (mem->buffer, buffer, len);
+	g_byte_array_append (mem->buffer, (unsigned char *) buffer, len);
 	
 	g_mime_stream_construct (GMIME_STREAM (mem), 0, -1);
 	
@@ -352,13 +346,32 @@ g_mime_stream_mem_new_with_buffer (const char *buffer, size_t len)
 }
 
 
+/**
+ * g_mime_stream_mem_get_byte_array:
+ * @mem: memory stream
+ *
+ * Gets the byte array from the memory stream.
+ *
+ * Returns the byte array from the memory stream.
+ **/
+GByteArray *
+g_mime_stream_mem_get_byte_array (GMimeStreamMem *mem)
+{
+	g_return_val_if_fail (GMIME_IS_STREAM_MEM (mem), NULL);
+	
+	return mem->buffer;
+}
+
+
 /**
  * g_mime_stream_mem_set_byte_array:
  * @mem: memory stream
  * @array: stream data
  *
- * Sets the byte array on the memory stream. Note: The memory stream
- * is not responsible for freeing the byte array.
+ * Sets the byte array on the memory stream.
+ *
+ * Note: The memory stream is not responsible for freeing the byte
+ * array.
  **/
 void
 g_mime_stream_mem_set_byte_array (GMimeStreamMem *mem, GByteArray *array)
@@ -368,7 +381,7 @@ g_mime_stream_mem_set_byte_array (GMimeStreamMem *mem, GByteArray *array)
 	g_return_if_fail (GMIME_IS_STREAM_MEM (mem));
 	g_return_if_fail (array != NULL);
 	
-	if (mem->buffer)
+	if (mem->owner && mem->buffer)
 		g_byte_array_free (mem->buffer, TRUE);
 	
 	mem->buffer = array;
@@ -380,3 +393,40 @@ g_mime_stream_mem_set_byte_array (GMimeStreamMem *mem, GByteArray *array)
 	stream->bound_start = 0;
 	stream->bound_end = -1;
 }
+
+
+/**
+ * g_mime_stream_mem_get_owner:
+ * @mem: memory stream
+ *
+ * Gets whether or not @mem owns the backend memory buffer.
+ *
+ * Returns %TRUE if @mem owns the backend memory buffer or %FALSE
+ * otherwise.
+ **/
+gboolean
+g_mime_stream_mem_get_owner (GMimeStreamMem *mem)
+{
+	g_return_val_if_fail (GMIME_IS_STREAM_MEM (mem), FALSE);
+	
+	return mem->owner;
+}
+
+
+/**
+ * g_mime_stream_mem_set_owner:
+ * @mem: memory stream
+ * @owner: owner
+ *
+ * Sets whether or not @mem owns the backend memory buffer.
+ *
+ * Note: @owner should be %TRUE if the stream should free the backend
+ * memory buffer when destroyed or %FALSE otherwise.
+ **/
+void
+g_mime_stream_mem_set_owner (GMimeStreamMem *mem, gboolean owner)
+{
+	g_return_if_fail (GMIME_IS_STREAM_MEM (mem));
+	
+	mem->owner = owner;
+}
